from .models import *
from django.contrib.auth.models import Permission,Group
from django.contrib.auth.hashers import make_password
from rest_framework_simplejwt.serializers import TokenObtainPairSerializer
from rest_framework import exceptions,serializers

class MyTokenObtainPairSerializer(TokenObtainPairSerializer):
    def validate(self,attr):
        data = super().validate(attr)
        data['id'] = self.user.id 
        data['first_name'] = self.user.first_name
        data['last_name'] = self.user.last_name
        data['email'] = self.user.email
        data['phone'] = str(self.user.phone)
        try:
            data['profile_image'] = self.user.profile_image.url
        except Exception as e:
            data['profile_image'] = 'null'
        
        token = self.get_token(self.user)
        if self.user:
            if self.user.company.is_active & self.user.is_active:
                return data
            raise exceptions.AuthenticationFailed('Account is not activated')
        raise exceptions.AuthenticationFailed()

class PermissionSerializer(serializers.ModelSerializer):
    class Meta:
        model = Permission
        fields = '__all__'
        
        
class GroupSerializer(serializers.ModelSerializer):
    class Meta:
        model = Group
        fields = '__all__'
        
class UserGroupSerializer(serializers.ModelSerializer):
    permissions = PermissionSerializer(many=True,read_only=True)
    class Meta:
        model = Group
        fields = '__all__'
        
        
class UserSerializer(serializers.ModelSerializer):
    groups = GroupSerializer(Group.objects.all(),many=True,read_only=True)
    class Meta:
        model = User
        read_only_fields = ["is_superuser"]
        fields = '__all__'
        extra_kwargs = {'password': {'write_only': True}}
        
    def create(self, validated_data):
        validated_data.pop('groups', None)
        validated_data.pop('user_permissions', None)
        return super(UserSerializer, self).create(validated_data)
    
       
    
    
    
class ProfileSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ('id','first_name','last_name','email','phone','profile_image')

        
        
    
        

        



from django.contrib.auth.base_user import BaseUserManager
from django.db import models
from django.contrib.auth.models import AbstractUser
from phonenumber_field.modelfields import PhoneNumberField
from django.contrib.auth.hashers import make_password
from django.contrib.auth.models import Group
from django.core.mail import EmailMultiAlternatives
from django.dispatch import receiver
from django.template.loader import render_to_string
from django.urls import reverse

from django_rest_passwordreset.signals import reset_password_token_created

class UserManager(BaseUserManager):   

    def create_user(self, email, password, **extra_fields):
        
        if not email:
            raise ValueError('The given email must be set')
        
        email = self.normalize_email(email)
        user = self.model(email=email, **extra_fields)
        user.password = make_password(password)
        user.save()
        return user

    def create_superuser(self, email, password, **extra_fields):
        """
        Create and save a SuperUser with the given email and password.
        """
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)
        extra_fields.setdefault('is_active', True)

        if extra_fields.get('is_staff') is not True:
            raise ValueError('Superuser must have is_staff=True.')
        if extra_fields.get('is_superuser') is not True:
            raise ValueError('Superuser must have is_superuser=True.')
        return self.create_user(email, password, **extra_fields)

# User model
class User(AbstractUser):
    class Meta:
        db_table = "user" 
        permissions = (
            ("can_view_unrealized", "Can View Unrealized Position Summary"),
            ("can_view_realized", "Can View Realized Position Summary"),
        )
    username = None    
    email = models.EmailField(('email'),max_length=100, unique=True)
    profile_image = models.ImageField(upload_to='images/profile_images/%Y/%m',blank=True,null=True)
    phone = PhoneNumberField(blank=True,null=True)
    is_active = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []
    
    objects = UserManager()
    
    
    
    
@receiver(reset_password_token_created)
def password_reset_token_created(sender, instance, reset_password_token, *args, **kwargs):
    """
    Handles password reset tokens
    When a token is created, an e-mail needs to be sent to the user
    :param sender: View Class that sent the signal
    :param instance: View Instance that sent the signal
    :param reset_password_token: Token Model Object
    :param args:
    :param kwargs:
    :return:
    """
    # send an e-mail to the user
    context = {
        'current_user': reset_password_token.user,
        'email': reset_password_token.user.email,
        'reset_password_url': "{}?token={}".format(
            'http://54.254.61.165/auth/password_reset',
            reset_password_token.key)
    }

    # render email text
    email_html_message = render_to_string('email/user_reset_password.html', context)
    email_plaintext_message = render_to_string('email/user_reset_password.txt', context)

    msg = EmailMultiAlternatives(
        # title:
        "Password Reset for {title}".format(title="Some website title"),
        # message:
        email_plaintext_message,
        # from:
        "noreply@somehost.local",
        # to:
        [reset_password_token.user.email]
    )
    msg.attach_alternative(email_html_message, "text/html")
    msg.send()


from .serializers import *
from .models import *
from rest_framework import viewsets,pagination
from rest_framework.permissions import AllowAny,IsAuthenticated
from django.contrib.auth.models import Group, Permission
from rest_framework.response import Response
from rest_framework_simplejwt.views import TokenObtainPairView
from rest_framework.decorators import api_view,permission_classes
from django.contrib.auth.hashers import make_password,check_password
from rest_framework.views import APIView
from rest_framework.exceptions import NotFound  

class LoginView(TokenObtainPairView):
    serializer_class = MyTokenObtainPairSerializer
    permission_classes = [AllowAny]
    
    def post(self, request):
        try:
            serializer = self.serializer_class(
                data=request.data, 
                context={'request': request}
                )           
                
            serializer.is_valid(raise_exception=True)
            return Response({
                "success":True,
                "data": serializer.validated_data
            }
            )
        except Exception as e:            
            return Response({"success":False,"error": e.detail })
        
        
class CustomPagination(pagination.PageNumberPagination): 
    page_query_param = 'page'
    page_size_query_param = 'limit'
    
    def paginate_queryset(self, queryset, request, view=None):
        """Checking NotFound exception"""
        try:
            return super(CustomPagination, self).paginate_queryset(queryset, request, view=view)
        except NotFound:  # intercept NotFound exception
            return list()
    
    def get_paginated_response(self, data):
        try:           
            next = self.page.next_page_number()
        except Exception as e:
            next = None
        try:           
            previous = self.page.previous_page_number()
        except Exception as e:
            previous = None
        return Response({            
            'next': next,
            'previous': previous,
            'current_page': self.page.number,
            'total_object': self.page.paginator.count,
            'total_page': self.page.paginator.num_pages,
            'results': data
        })
    
paginator = CustomPagination() 
    
class UserViewSet(viewsets.ModelViewSet):
    queryset=User.objects.all()
    serializer_class=UserSerializer
    pagination_class = CustomPagination
    
    def list(self, request):
        users=User.objects.all().exclude(id=request.user.id).order_by('-id')
        if self.request.query_params.get('limit') == '0':
            serializer=UserSerializer(users, many=True)
            return Response({'results':serializer.data})
        else:
            page = self.paginate_queryset(users)
            serializer=UserSerializer(page, many=True)
            return self.get_paginated_response(serializer.data)
    
    
    def create(self, request):
        try:
            data=request.data
            serializer = UserSerializer(data=request.data)
            serializer.is_valid(raise_exception=True)
            user = serializer.save()
            for i in data['groups']:
                user.groups.add(i)
                user.save()
            
        except Exception as e:
            print(e)
            return Response({"success": False,"error": list(serializer.errors.values())[0][0] })
        else:
            return Response({"success": True,"data":serializer.data})
    
    def partial_update(self, request, pk=None):
        try:
            data=request.data
            instance = User.objects.get(id=pk)
            serializer = UserSerializer(instance=instance, data=request.data,partial=True)
            serializer.is_valid(raise_exception=True)
            user = serializer.save()
            user.groups.clear()
            for i in data['groups']:
                user.groups.add(i)
                user.save()
        except Exception as e:
            return Response({"success": False,"error": list(serializer.errors.values())[0][0] })
        else:
            return Response({"success": True,"data":serializer.data})
        

    

@api_view(['PATCH'])
@permission_classes([IsAuthenticated])  
def update_profile(request):
    try:        
        serializer = ProfileSerializer(request.user, data=request.data,partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
    except Exception as e:
        print(e)
        return Response({"success": False,"error": list(serializer.errors.values())[0][0] })
    else:
        return Response({"success": True,"data":serializer.data})
          
@api_view(['PUT'])
@permission_classes([IsAuthenticated])  
def change_password(request):
    try:   
        user = request.user
        checkPassword = check_password(request.data['old_password'], user.password)
        if checkPassword:        
            user.password = make_password(request.data['new_password'])
            user.save() 
        else:
            return Response({"success": False,'error':'wrong old password'})
    except Exception as e:
        return Response({"success": False,'error': 'Something Wrong!'})
    else:
        return Response({"success": True})
        

    queryset=CustomGroup.objects.all()
    serializer_class=CustomGroupSerializer
    pagination_class = CustomPagination
    
    def list(self, request):
        groups=CustomGroup.objects.filter(company=self.request.user.company).order_by('-id')
        if self.request.query_params.get('limit') == '0':
            serializer=CustomGroupSerializer(groups, many=True)
            return Response({'results':serializer.data})
        else:
            page = self.paginate_queryset(groups)
            serializer=CustomGroupSerializer(page, many=True)
            return self.get_paginated_response(serializer.data)
    
    def create(self, request):
        try:
            data = request.data        
            group = Group.objects.create(name=data['name'])
            for x in data['permissions']:
                permission = Permission.objects.get(pk=x)
                group.permissions.add(permission)
                group.save()
            serializer = CustomGroupSerializer(data={'group_id':group.pk,'company':self.request.user.company.pk})
            serializer.is_valid(raise_exception=True)
            serializer.save()
        except Exception as e:
            print(e)
            return Response({"success": False,"error": list(serializer.errors.values())[0][0] })
        else:
            return Response({"success": True,"data":serializer.data})
    
    def update(self, request, pk=None):
        try:
            data = request.data
            print(data)
            cgroups = CustomGroup.objects.get(id=pk)
            group = Group.objects.get(id=cgroups.group.id)
            group.permissions.clear()
            for x in data['permissions']:
                permission = Permission.objects.get(pk=x)
                group.permissions.add(permission)
                group.save()
            
        except Exception as e:
            return Response({"success": False,"error": 'Something Wrong!' })
        else:
            return Response({"success": True})
    
    
class PermissionViewSet(viewsets.ModelViewSet):
    queryset=Permission.objects.filter().exclude(codename__in=["add_logentry","change_logentry","delete_logentry","view_logentry","add_permission","change_permission","delete_permission","view_permission","add_customgroup","change_customgroup","delete_customgroup","view_customgroup","add_contenttype","change_contenttype","delete_contenttype","view_contenttype","add_session","change_session","delete_session","view_session"])
    serializer_class=PermissionSerializer
    
    
class UserRoleApi(APIView):
    permission_classes = [IsAuthenticated]
    
    def get(self,request):
        permissions = []
        group = request.user.groups.first()
        serializer = UserGroupSerializer(group)
        for i in serializer.data['permissions']:
            permissions.append(i['codename'])
                   
        return Response({'permissions':permissions})